@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: dispatcher.cc,v 1.8 2005/01/06 17:42:56 murrayc Exp $ */
+/* $Id: dispatcher.cc,v 1.12.4.1 2006/05/10 08:05:02 murrayc Exp $ */
 
 /* Copyright 2002 The gtkmm Development Team
  *
@@ -30,6 +30,18 @@
 
 #ifndef G_OS_WIN32
 #include <unistd.h>
+
+#if defined(_tru64) //TODO: Use the real define
+//EINTR is not defined on Tru64
+//I have tried including these
+//#include <sys/types.h>
+//#include <sys/statvfs.h>
+//#include <signal.h>
+  #ifndef EINTR
+  #define EINTR 0
+  #endif
+#endif
+
 #else
 #include <windows.h>
 #include <io.h>
@@ -117,7 +129,7 @@ static void fd_close_and_invalidate(int& fd)
 namespace Glib
 {
 
-class DispatchNotifier
+class DispatchNotifier : public sigc::trackable
 {
 public:
   ~DispatchNotifier();
@@ -176,8 +188,10 @@ DispatchNotifier::DispatchNotifier(const Glib::RefPtr<MainContext>& context)
 {
   create_pipe();
 
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
 #ifdef G_OS_WIN32
     conn_io_handler_ = context_->signal_io().connect(
         sigc::mem_fun(*this, &DispatchNotifier::pipe_io_handler),
@@ -187,6 +201,7 @@ DispatchNotifier::DispatchNotifier(const Glib::RefPtr<MainContext>& context)
         sigc::mem_fun(*this, &DispatchNotifier::pipe_io_handler),
         fd_receiver_, Glib::IO_IN);
 #endif /* !G_OS_WIN32 */
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   }
   catch(...)
   {
@@ -197,6 +212,7 @@ DispatchNotifier::DispatchNotifier(const Glib::RefPtr<MainContext>& context)
 
     throw;
   }
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
 }
 
 DispatchNotifier::~DispatchNotifier()
@@ -219,20 +235,29 @@ void DispatchNotifier::create_pipe()
 
   if(!fd_receiver_)
   {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     GError* const error = g_error_new(G_FILE_ERROR, G_FILE_ERROR_FAILED,
                                       "Failed to create event for inter-thread communication: %s",
                                       g_win32_error_message(GetLastError()));
     throw Glib::FileError(error);
+#else 
+    return; //TODO: Provide an alternative to the exception.
+#endif //GLIBMM_EXCEPTIONS_ENABLED
   }
+
 #else /* !G_OS_WIN32 */
   int filedes[2] = { -1, -1 };
 
   if(pipe(filedes) < 0)
   {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     GError* const error = g_error_new(G_FILE_ERROR, g_file_error_from_errno(errno),
                                       "Failed to create pipe for inter-thread communication: %s",
                                       g_strerror(errno));
     throw Glib::FileError(error);
+#else 
+    return; //TODO: Provide an alternative to the exception.
+#endif //GLIBMM_EXCEPTIONS_ENABLED
   }
 
   fd_set_close_on_exec(filedes[0]);
@@ -341,14 +366,18 @@ bool DispatchNotifier::pipe_io_handler(Glib::IOCondition)
     // Actually, we wouldn't need the try/catch block because the Glib::Source
     // C callback already does it for us.  However, we do it anyway because the
     // default return value is 'false', which is not what we want.
+    #ifdef GLIBMM_EXCEPTIONS_ENABLED
     try
     {
+    #endif //GLIBMM_EXCEPTIONS_ENABLED
       data.dispatcher->signal_();
+    #ifdef GLIBMM_EXCEPTIONS_ENABLED
     }
     catch(...)
     {
       Glib::exception_handlers_invoke();
     }
+    #endif //GLIBMM_EXCEPTIONS_ENABLED
   }
 #else /* !G_OS_WIN32 */
   DispatchNotifyData data;
@@ -378,14 +407,18 @@ bool DispatchNotifier::pipe_io_handler(Glib::IOCondition)
   // Actually, we wouldn't need the try/catch block because the Glib::Source
   // C callback already does it for us.  However, we do it anyway because the
   // default return value is 'false', which is not what we want.
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
     data.dispatcher->signal_(); // emit
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   }
   catch(...)
   {
     Glib::exception_handlers_invoke();
   }
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
 #endif /* !G_OS_WIN32 */
 
   return true;
