@@ -1,5 +1,5 @@
 // -*- c++ -*-
-/* $Id: ustring.cc,v 1.4 2005/01/06 17:42:56 murrayc Exp $ */
+/* $Id: ustring.cc,v 1.6.2.1 2006/05/11 17:29:32 murrayc Exp $ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
@@ -35,7 +35,6 @@ namespace
 
 using Glib::ustring;
 
-
 // Little helper to make the conversion from gunichar to UTF-8 a one-liner.
 //
 struct UnicharToUtf8
@@ -141,7 +140,7 @@ ustring::size_type utf8_char_offset(const std::string& str, ustring::size_type o
 // Helper to implement ustring::find_first_of() and find_first_not_of().
 // Returns the UTF-8 character offset, or ustring::npos if not found.
 //
-ustring::size_type utf8_find_first_of(const std::string& str, ustring::size_type offset,
+static ustring::size_type utf8_find_first_of(const std::string& str, ustring::size_type offset,
                                       const char* utf8_match, long utf8_match_size,
                                       bool find_not_of)
 {
@@ -177,7 +176,7 @@ ustring::size_type utf8_find_first_of(const std::string& str, ustring::size_type
 // Helper to implement ustring::find_last_of() and find_last_not_of().
 // Returns the UTF-8 character offset, or ustring::npos if not found.
 //
-ustring::size_type utf8_find_last_of(const std::string& str, ustring::size_type offset,
+static ustring::size_type utf8_find_last_of(const std::string& str, ustring::size_type offset,
                                      const char* utf8_match, long utf8_match_size,
                                      bool find_not_of)
 {
@@ -215,6 +214,12 @@ ustring::size_type utf8_find_last_of(const std::string& str, ustring::size_type
 namespace Glib
 {
 
+#ifndef GLIBMM_HAVE_ALLOWS_STATIC_INLINE_NPOS
+// Initialize static member here, 
+// because the compiler did not allow us do it inline.
+const ustring::size_type ustring::npos = std::string::npos;
+#endif
+
 // We need our own version of g_utf8_get_char(), because the std::string
 // iterator is not necessarily a plain pointer (it's in fact not in GCC's
 // libstdc++-v3).  Copying the UTF-8 data into a temporary buffer isn't an
@@ -1188,13 +1193,25 @@ std::istream& operator>>(std::istream& is, Glib::ustring& utf8_string)
 {
   std::string locale_string;
   is >> locale_string;
+
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   utf8_string = Glib::locale_to_utf8(locale_string);
+  #else
+  std::auto_ptr<Glib::Error> error; //TODO: Check this? 
+  utf8_string = Glib::locale_to_utf8(locale_string, error);
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
   return is;
 }
 
 std::ostream& operator<<(std::ostream& os, const Glib::ustring& utf8_string)
 {
+  #ifdef GLIBMM_EXCEPTIONS_ENABLED
   os << Glib::locale_from_utf8(utf8_string);
+  #else
+  std::auto_ptr<Glib::Error> error; //TODO: Check this? 
+  os << Glib::locale_from_utf8(utf8_string, error);
+  #endif //GLIBMM_EXCEPTIONS_ENABLED
+
   return os;
 }
 
