@@ -54,13 +54,14 @@ use warnings;
 
 #####################################
 
+$DocsParser::CurrentFile = "";
 
 $DocsParser::refAppendTo = undef; # string reference to store the data into
 $DocsParser::currentParam = undef;
 
-$DocsParser::objCurrentFunction = 0; #Function
+$DocsParser::objCurrentFunction = undef; #Function
 %DocsParser::hasharrayFunctions = (); #Function elements
-$DocsParser::bOverride = 0; #First we parse the C docs, then we parse the C++ override docs.
+#~ $DocsParser::bOverride = 0; #First we parse the C docs, then we parse the C++ override docs.
 
 $DocsParser::commentStart = "  /** ";
 $DocsParser::commentMiddleStart = "   * ";
@@ -69,31 +70,41 @@ $DocsParser::commentEnd = "   */";
 sub read_defs($$$)
 {
   my ($path, $filename, $filename_override) = @_;
-   
-  # check that the file is there.
-  my $filepath = "$path/$filename";
-  if ( ! -r $filepath)
-  {
-     print "DocsParser.pm: Error: can't read defs file $filename\n";
-     return;
-  }
-
-  my $filepath_override = "$path/$filename_override";
-  if ( ! -r $filepath_override)
-  {
-     print "DocsParser.pm: Error: can't read defs file $filename_override\n";
-     return;
-  }
- 
-  my $objParser = new XML::Parser();
+  
+  my $objParser = new XML::Parser(ErrorContext => 0);
   $objParser->setHandlers(Start => \&parse_on_start, End => \&parse_on_end, Char => \&parse_on_cdata);
+  
+  # C documentation:
+  $DocsParser::CurrentFile = "$path/$filename";
+  if ( ! -r $DocsParser::CurrentFile)
+  {
+     print "DocsParser.pm: Warning: Can't read file \"" . $DocsParser::CurrentFile . "\".\n";
+     return;
+  }
+  # Parse
+  eval { $objParser->parsefile($DocsParser::CurrentFile) };
+  if( $@ )
+  {
+    $@ =~ s/at \/.*?$//s;
+    print "\nError in \"" . $DocsParser::CurrentFile . "\":$@\n";
+    return;
+  }
 
-  # Parse the C docs:
-  $objParser->parsefile($filepath);
-
-  # Parse the C++ overide docs:
-  $DocsParser::bOverride = 1; #The callbacks will act differently when this is set.
-  $objParser->parsefile($filepath_override);
+  # C++ overide documentation:
+  $DocsParser::CurrentFile = "$path/$filename_override";
+  if ( ! -r $DocsParser::CurrentFile)
+  {
+     print "DocsParser.pm: Warning: Can't read file \"" . $DocsParser::CurrentFile . "\".\n";
+     return;
+  }
+  # Parse
+  eval { $objParser->parsefile($DocsParser::CurrentFile) };
+  if( $@ )
+  {
+    $@ =~ s/at \/.*?$//s;
+    print "\nError in \"" . $DocsParser::CurrentFile . "\":$@";
+    return;
+  }
 }
 
 sub parse_on_start($$%)
@@ -104,6 +115,11 @@ sub parse_on_start($$%)
 
   if($tag eq "function")
   {
+    if(defined $DocsParser::objCurrentFunction)
+    {
+      $objParser->xpcroak("\nClose a function tag before you open another one.");
+    }
+    
     my $functionName = $attr{name};
 
     #Reuse existing Function, if it exists:
@@ -113,14 +129,25 @@ sub parse_on_start($$%)
     {
       #Make a new one if necessary:
       $DocsParser::objCurrentFunction = Function::new_empty();
+      # The idea is to change the policy a bit:
+      # If a function is redefined in a later parsing run only values which are redefined
+      # will be overwritten. For the name this is trivial. The description is simply rewritten.
+      # Same goes for the return description and the class mapping. Only exception is the
+      # parameter list. Everytime we enter a <parameters> tag the list is emptied again.
+      $$DocsParser::objCurrentFunction{name} = $functionName;
+      $$DocsParser::objCurrentFunction{description} = "";
+      $$DocsParser::objCurrentFunction{param_names} = [];
+      $$DocsParser::objCurrentFunction{param_descriptions} = ();
+      $$DocsParser::objCurrentFunction{return_description} = "";
+      $$DocsParser::objCurrentFunction{mapped_class} = "";
+      # We don't need this any more, the only reference to this field is commented
+      # $$DocsParser::objCurrentFunction{description_overridden} = $DocsParser::bOverride;
     }
-
-    $$DocsParser::objCurrentFunction{name} = $functionName;
-    $$DocsParser::objCurrentFunction{description} = "";
+  }
+  elsif($tag eq "parameters")
+  {
     $$DocsParser::objCurrentFunction{param_names} = [];
     $$DocsParser::objCurrentFunction{param_descriptions} = ();
-    $$DocsParser::objCurrentFunction{return_description} = "";
-    $$DocsParser::objCurrentFunction{description_overridden} = $DocsParser::bOverride;
   }
   elsif($tag eq "parameter")
   {
@@ -129,6 +156,7 @@ sub parse_on_start($$%)
   }
   elsif($tag eq "description")
   {
+    $$DocsParser::objCurrentFunction{description} = "";
     # Set destination for parse_on_cdata().
     $DocsParser::refAppendTo = \$$DocsParser::objCurrentFunction{description};
   }
@@ -140,9 +168,18 @@ sub parse_on_start($$%)
   }
   elsif($tag eq "return")
   {
+    $$DocsParser::objCurrentFunction{return_description} = "";
     # Set destination for parse_on_cdata().
     $DocsParser::refAppendTo = \$$DocsParser::objCurrentFunction{return_description};
   }
+  elsif($tag eq "mapping")
+  {
+    $$DocsParser::objCurrentFunction{mapped_class} = $attr{class};
+  }
+  elsif($tag ne "root")
+  {
+    $objParser->xpcroak("\nUnknown tag \"$tag\".");
+  }
 }
 
 
@@ -195,38 +232,45 @@ sub parse_on_cdata($$)
 
 
 # $strCommentBlock lookup_documentation($strFunctionName)
-sub lookup_documentation($)
+sub lookup_documentation($$)
 {
-  my ($functionName) = @_;
-   
+  my ($functionName, $deprecation_docs) = @_;
+
   my $objFunction = $DocsParser::hasharrayFunctions{$functionName};
   if(!$objFunction)
   {
     #print "DocsParser.pm: Warning: function not found: $functionName\n";
     return ""
   }
-  
+
   my $text = $$objFunction{description};
 
   if(length($text) eq 0)
   {
-    print "DocsParser.pm: Warning: No C docs for function:$functionName :\n";
+    print "DocsParser.pm: Warning: No C docs for function: \"$functionName\"\n";
   }
-  
-      
+
+
   DocsParser::convert_docs_to_cpp($objFunction, \$text);
+
+  #Add note about deprecation if we have specified that in our _WRAP_METHOD() call:
+  if($deprecation_docs ne "")
+  {
+    $text .= "\n\@deprecated $deprecation_docs";
+  }
+
   DocsParser::append_parameter_docs($objFunction, \$text);
   DocsParser::append_return_docs($objFunction, \$text);
 
-    
+
   # Escape the space after "i.e." or "e.g." in the brief description.
   $text =~ s/^([^.]*\b(?:i\.e\.|e\.g\.))\s/$1\\ /;
-      
+
   # Convert to Doxygen-style comment.
   $text =~ s/\n/\n${DocsParser::commentMiddleStart}/g;
   $text =  $DocsParser::commentStart . $text;
   $text .= "\n${DocsParser::commentEnd}\n";
-      
+
   return $text;
 }
 
@@ -240,13 +284,17 @@ sub append_parameter_docs($$)
 
   # Strip first parameter if this is a method.
   my $defs_method = GtkDefs::lookup_method_dont_mark($$obj_function{name});
-  shift(@param_names) if($defs_method && $$defs_method{class} ne "");
+  # the second alternative is for use with method-mappings meaning:
+  # this function is mapped into this Gtk::class
+  shift(@param_names) if(($defs_method && $$defs_method{class} ne "") ||
+                         ($$obj_function{mapped_class} ne ""));
 
   foreach my $param (@param_names)
   {
     my $desc = $$param_descriptions->{$param};
+    
+    $param =~ s/([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?/$1/g;
     DocsParser::convert_docs_to_cpp($obj_function, \$desc);
-
     if(length($desc) > 0)
     {
       $desc  .= '.' unless($desc =~ /(?:^|\.)$/);
@@ -275,7 +323,8 @@ sub convert_docs_to_cpp($$)
   # Chop off leading and trailing whitespace.
   $$text =~ s/^\s+//;
   $$text =~ s/\s+$//;
-
+# HagenM: this is the only reference to $$obj_function{description_overridden}
+# and it seems not to be in use.
 #  if(!$$obj_function{description_overridden})
 #  {
     # Convert C documentation to C++.
@@ -300,11 +349,9 @@ sub convert_tags_to_doxygen($)
     s"&lt;(/?)function&gt;"<$1tt>"g;
 
     # Some argument names are suffixed by "_" -- strip this.
-    s" ?\@([_a-z]*[a-z])_?\b" \@a $1 "g;
-    s"^Note ?\d?: "\@note "mg;
-
     # gtk-doc uses @thearg, but doxygen uses @a thearg.
-    #s"\@"\@amurrayc "g;
+    s" ?\@([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?\b" \@a $1 "g;
+    s"^Note ?\d?: "\@note "mg;
 
     s"&lt;/?programlisting&gt;""g;
     s"&lt;informalexample&gt;"\@code"g;
@@ -317,6 +364,14 @@ sub convert_tags_to_doxygen($)
     # Remove all para tags (from tmpl sgml files).
     s"&lt;/?para&gt;""g;
 
+    # Use our doxgen since/newin tags:
+    # TODO: Do this generically, regardless of the number:
+    s"Since: 2\.2"\@newin2p2"mg;
+    s"Since: 2\.4"\@newin2p4"mg;
+    s"Since: 2\.6"\@newin2p6"mg;
+    s"Since: 2\.8"\@newin2p8"mg;
+    s"Since: 2\.10"\@newin2p10"mg;
+
     s"\b-&gt;\b"->"g;
 
     # Doxygen is too dumb to handle &mdash;
